﻿namespace TorrentHarvester
{
    using Microsoft.VisualBasic;
    using Microsoft.VisualBasic.CompilerServices;
    using System;
    using System.Collections.Specialized;
    using System.Diagnostics;
    using System.IO;
    using System.Windows.Forms;

    public class CCommand
    {
        private EActionType _ActionType;
        private NameValueCollection Variable;
        private const string VARIABLE_CLIPBOARD_TEXT = "text";
        private const string VARIABLE_CREATEFILE_FILENAME = "filename";
        private const string VARIABLE_CREATEFILE_TEXT = "text";
        private const string VARIABLE_DOWNLOAD_FILENAME = "filename";
        private const string VARIABLE_DOWNLOAD_URL = "url";
        private const string VARIABLE_EXECUTE_FILENAME = "filename";
        private const string VARIABLE_LAUNCHURL_URL = "url";

        public CCommand()
        {
            this.Variable = new NameValueCollection();
        }

        public CCommand(CXMLNode oXMLNode)
        {
            this.Variable = new NameValueCollection();
            this.Load(oXMLNode);
        }

        public void ClearVariables()
        {
            this.Variable = new NameValueCollection();
        }

        public void Init_Clipboard()
        {
            this.ClearVariables();
            this.Variable["text"] = "";
        }

        public void Init_CreateFile()
        {
            this.ClearVariables();
            this.Variable["filename"] = "";
            this.Variable["text"] = "";
        }

        public void Init_Download()
        {
            this.ClearVariables();
            this.Variable["filename"] = "";
            this.Variable["url"] = "";
            this.Variable["postdata"] = "";
        }

        public void Init_Execute()
        {
            this.ClearVariables();
            this.Variable["filename"] = "";
        }

        public void Init_LaunchURL()
        {
            this.ClearVariables();
            this.Variable["url"] = "";
        }

        public void Load(CXMLNode oXMLNode)
        {
            this.Name = oXMLNode.Name;
            StringEnumerator enumerator = oXMLNode.AttributeNames.GetEnumerator();
            while (enumerator.MoveNext())
            {
                string current = enumerator.Current;
                this.Variable[current] = oXMLNode[current, ""];
            }
        }

        public bool Perform(CFile oFile)
        {
            bool flag = false;
            switch (this.ActionType)
            {
                case EActionType.actClipboard:
                    flag = this.Perform_Clipboard(oFile);
                    break;
                case EActionType.actLaunchURL:
                    flag = this.Perform_LaunchURL(oFile);
                    break;
                case EActionType.actExecute:
                    flag = this.Perform_Execute(oFile);
                    break;
                case EActionType.actCreateFile:
                    flag = this.Perform_CreateFile(oFile);
                    break;
                case EActionType.actDownload:
                    flag = this.Perform_Download(oFile);
                    break;
            }
            return flag;
        }

        private bool Perform_Clipboard(CFile oFile)
        {
            bool flag = false;
            try
            {
                Clipboard.SetDataObject(oFile.Translate(this.Variable["text"]));
                flag = true;
            }
            catch (Exception exception1)
            {
                ProjectData.SetProjectError(exception1);
                flag = false;
                ProjectData.ClearProjectError();
            }
            return flag;
        }

        private bool Perform_CreateFile(CFile oFile)
        {
            bool flag = false;
            try
            {
                string str = modGlobal.g_oStrings.FixFilename(oFile.Translate(this.Variable["filename"]));
                string strText = this.Variable["text"];
                strText = Strings.Replace(strText.Substring(1), StringType.FromChar(strText[0]), "\r\n", 1, -1, CompareMethod.Binary);
                strText = oFile.Translate(strText);
                StreamWriter writer = new StreamWriter(modGlobal.DownloadFolder + str);
                writer.Write(strText);
                writer.Close();
                flag = true;
            }
            catch (Exception exception1)
            {
                ProjectData.SetProjectError(exception1);
                flag = false;
                ProjectData.ClearProjectError();
            }
            return flag;
        }

        private bool Perform_Download(CFile oFile)
        {
            bool flag = false;
            try
            {
                string cookie;
                string strFilename = modGlobal.g_oStrings.FixFilename(oFile.Translate(this.Variable["filename"]));
                string strURL = oFile.Translate(oFile.Page.Engine.FullURL(oFile.Translate(this.Variable["url"])));
                string strTitle = oFile["Title"];
                if (oFile.Page.Engine.Login == null)
                {
                    cookie = "";
                }
                else
                {
                    cookie = oFile.Page.Engine.Login.Cookie;
                }
                flag = modGlobal.Download(strTitle, strURL, strFilename, modGlobal.DownloadFolder, cookie);
            }
            catch (Exception exception1)
            {
                ProjectData.SetProjectError(exception1);
                flag = false;
                ProjectData.ClearProjectError();
            }
            return flag;
        }

        private bool Perform_Execute(CFile oFile)
        {
            bool flag = false;
            try
            {
                bool flag2 = true;
                string str = modGlobal.g_oStrings.FixFilename(oFile.Translate(this.Variable["filename"]));
                if (modGlobal.g_oSettings.AskBeforeExecute)
                {
                    flag2 = Interaction.MsgBox("Execute File", MsgBoxStyle.YesNo, "Execute: " + str) == MsgBoxResult.Yes;
                }
                if (flag2)
                {
                    Process.Start(modGlobal.DownloadFolder + str);
                }
                flag = true;
            }
            catch (Exception exception1)
            {
                ProjectData.SetProjectError(exception1);
                flag = false;
                ProjectData.ClearProjectError();
            }
            return flag;
        }

        private bool Perform_LaunchURL(CFile oFile)
        {
            bool flag = false;
            try
            {
                modGlobal.LaunchBrowser(oFile.Page.Engine.FullURL(oFile.Translate(this.Variable["url"])));
                flag = true;
            }
            catch (Exception exception1)
            {
                ProjectData.SetProjectError(exception1);
                flag = false;
                ProjectData.ClearProjectError();
            }
            return flag;
        }

        public void Set_Clipboard(string strText)
        {
            this.ClearVariables();
            this._ActionType = EActionType.actClipboard;
            this.Variable["text"] = strText;
        }

        public void Set_CreateFile(string strFilename, string strText)
        {
            this.ClearVariables();
            this._ActionType = EActionType.actCreateFile;
            this.Variable["filename"] = strFilename;
            this.Variable["text"] = strText;
        }

        public void Set_Download(string strFilename, string strURL, string strPostData)
        {
            this.ClearVariables();
            this._ActionType = EActionType.actDownload;
            this.Variable["filename"] = strFilename;
            this.Variable["url"] = strURL;
            this.Variable["postdata"] = strPostData;
        }

        public void Set_Execute(string strFilename)
        {
            this.ClearVariables();
            this._ActionType = EActionType.actExecute;
            this.Variable["filename"] = strFilename;
        }

        public void Set_LaunchURL(string strFilename)
        {
            this.ClearVariables();
            this._ActionType = EActionType.actLaunchURL;
            this.Variable["url"] = strFilename;
        }

        public override string ToString()
        {
            return ("\t" + this.Name);
        }

        public EActionType ActionType
        {
            get
            {
                return this._ActionType;
            }
            set
            {
                this._ActionType = value;
                switch (this._ActionType)
                {
                    case EActionType.actClipboard:
                        this.Init_Clipboard();
                        break;

                    case EActionType.actLaunchURL:
                        this.Init_LaunchURL();
                        break;

                    case EActionType.actExecute:
                        this.Init_Execute();
                        break;

                    case EActionType.actCreateFile:
                        this.Init_CreateFile();
                        break;

                    case EActionType.actDownload:
                        this.Init_Download();
                        break;
                }
            }
        }

        public string Name
        {
            get
            {
                string str2 = "";
                switch (this.ActionType)
                {
                    case EActionType.actClipboard:
                        str2 = "clipboard";
                        break;
                    case EActionType.actLaunchURL:
                        str2 = "launchurl";
                        break;
                    case EActionType.actExecute:
                        str2 = "execute";
                        break;
                    case EActionType.actCreateFile:
                        str2 = "createfile";
                        break;
                    case EActionType.actDownload:
                        str2 = "download";
                        break;
                }
                return str2;
            }
            set
            {
                string sLeft = value;
                if (StringType.StrCmp(sLeft, "clipboard", false) == 0)
                {
                    this.ActionType = EActionType.actClipboard;
                }
                else if (StringType.StrCmp(sLeft, "launchurl", false) == 0)
                {
                    this.ActionType = EActionType.actLaunchURL;
                }
                else if (StringType.StrCmp(sLeft, "execute", false) == 0)
                {
                    this.ActionType = EActionType.actExecute;
                }
                else if (StringType.StrCmp(sLeft, "createfile", false) == 0)
                {
                    this.ActionType = EActionType.actCreateFile;
                }
                else if (StringType.StrCmp(sLeft, "download", false) == 0)
                {
                    this.ActionType = EActionType.actDownload;
                }
            }
        }

        public int NumVariables
        {
            get
            {
                return this.Variable.Count;
            }
        }

        public string this[int intIndex]
        {
            get
            {
                return this.Variable[intIndex];
            }
            set
            {
                this.Variable[this.Variable[intIndex]] = value;
            }
        }

        public string this[string strValue]
        {
            get
            {
                return this.Variable[strValue];
            }
            set
            {
                this.Variable[strValue] = value;
            }
        }

        public enum EActionType
        {
            actClipboard,
            actLaunchURL,
            actExecute,
            actCreateFile,
            actDownload
        }
    }
}

