﻿namespace TorrentHarvester
{
    using Microsoft.VisualBasic.CompilerServices;
    using System;
    using System.ComponentModel;
    using System.IO;
    using System.Net;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;
    using System.Threading;

    public class CDownloader
    {
        private delegate void DownloadErrorDelegate(string strMessage);
        public delegate void DownloadErrorEventHandler(string strMessage);
        private delegate void DownloadStatusDelegate(int PercentComplete, int BytesTransferred, int TotalBytes);
        public delegate void DownloadStatusEventHandler(int PercentComplete, int BytesTransferred, int TotalBytes);
        private delegate void GotFilenameDelegate(string strFilename);
        public delegate void GotFilenameEventHandler(string strFilename);

        private bool _cancel;
        private ISynchronizeInvoke _SynchronisingObject;
        public bool Closed;
        private AutoResetEvent ev = new AutoResetEvent(false);
        private bool m_bAppend;
        private string m_strCookie;
        private string m_strFilename;
        private string m_strFolder;
        private string m_strMethod;
        private string m_strPostData;
        private string m_strURL;
        private HTTPDownloadState mystate = new HTTPDownloadState();

        public event DownloadErrorEventHandler DownloadErrorEvent;
        public event DownloadStatusEventHandler DownloadStatusEvent;
        public event GotFilenameEventHandler GotFilenameEvent;

        public CDownloader(ISynchronizeInvoke objSynchronisingObject)
        {
            this._SynchronisingObject = objSynchronisingObject;
        }

        private void _Download()
        {
            this.Closed = false;
            try
            {
                int length = 0;
                HttpWebRequest webRequest = this.GetWebRequest();
                this.mystate.Response = (HttpWebResponse) webRequest.GetResponse();
                if (StringType.StrCmp(this.m_strFilename, "", false) == 0)
                {
                    this.m_strFilename = this.ResponseFilename(this.mystate.Response);
                    try
                    {
                        this._SynchronisingObject.Invoke(new GotFilenameDelegate(this.GotFilenameSub), new object[] { this.m_strFilename });
                    }
                    catch (Exception exception1)
                    {
                        ProjectData.SetProjectError(exception1);
                        if (this.GotFilenameEvent != null)
                        {
                            this.GotFilenameEvent(this.m_strFilename);
                        }
                        ProjectData.ClearProjectError();
                    }
                }
                this.m_strFilename = this.m_strFolder + this.m_strFilename;
                FileMode create = FileMode.Create;
                if (this.m_bAppend && File.Exists(this.m_strFilename))
                {
                    length = (int) new FileInfo(this.m_strFilename).Length;
                    if (length > 0)
                    {
                        this.mystate.Response.Close();
                        webRequest = this.GetWebRequest();
                        webRequest.AddRange(length, ((int) this.mystate.Response.ContentLength) - 1);
                        this.mystate.Response = (HttpWebResponse) webRequest.GetResponse();
                        create = FileMode.Append;
                        this.mystate.BytesRead = length;
                    }
                }
                this.mystate.Outputstream = new FileStream(this.m_strFilename, create);
                this.mystate.TotalBytes = length + ((int) this.mystate.Response.ContentLength);
                this.mystate.Inputstream = this.mystate.Response.GetResponseStream();
                this.mystate.Inputstream.BeginRead(this.mystate.Data, 0, this.mystate.Data.Length, new AsyncCallback(this.GotData), this.mystate);
                this.ev.WaitOne();
            }
            catch (Exception exception2)
            {
                ProjectData.SetProjectError(exception2);
                Exception exception = exception2;
                if (!this._cancel)
                {
                    try
                    {
                        this._SynchronisingObject.Invoke(new DownloadErrorDelegate(this.DownloadErrorSub), new object[] { exception.Message });
                    }
                    catch (Exception exception3)
                    {
                        ProjectData.SetProjectError(exception3);
                        if (this.DownloadErrorEvent != null)
                        {
                            this.DownloadErrorEvent(exception.Message);
                        }
                        ProjectData.ClearProjectError();
                    }
                }
                ProjectData.ClearProjectError();
            }
        }

        public void Cancel()
        {
            this._cancel = true;
            try
            {
                if (this.mystate.Outputstream != null)
                {
                    this.mystate.Outputstream.Close();
                    File.Delete(this.m_strFilename);
                }
            }
            catch (Exception exception1)
            {
                ProjectData.SetProjectError(exception1);
                ProjectData.ClearProjectError();
            }
        }

        private void Close(HTTPDownloadState mystate)
        {
            mystate.Outputstream.Close();
            if (!mystate.DontCloseInputStream && (mystate.Inputstream != null))
            {
                mystate.Inputstream.Close();
            }
            if (mystate.Response != null)
            {
                mystate.Response.Close();
            }
            this.ev.Set();
            this.Closed = true;
        }

        public void Download(string strURL, string strFolder, string strFilename, string strCookie, bool bAppend, [Optional, DefaultParameterValue(true)] bool bAsyncrhonous)
        {
            this._cancel = false;
            if (strURL.EndsWith(" POST"))
            {
                strURL = strURL.Substring(0, strURL.Length - 5);
                int index = strURL.IndexOf("?");
                if (index != -1)
                {
                    this.m_strPostData = strURL.Substring(index + 1);
                    strURL = strURL.Substring(0, index);
                }
                this.m_strMethod = "POST";
            }
            else
            {
                this.m_strMethod = "GET";
            }
            this.m_strURL = strURL;
            this.m_strFilename = strFilename;
            this.m_strFolder = strFolder;
            this.m_bAppend = bAppend;
            this.m_strCookie = strCookie;
            if (bAsyncrhonous)
            {
                Thread thread = new Thread(new ThreadStart(this._Download));
                thread.Start();
                thread = null;
            }
            else
            {
                this._Download();
            }
        }

        protected virtual void DownloadErrorSub(string strMessage)
        {
            if (this.DownloadErrorEvent != null)
            {
                this.DownloadErrorEvent(strMessage);
            }
        }

        protected virtual void DownloadStatusSub(int PercentComplete, int BytesTransferred, int TotalBytes)
        {
            if (this.DownloadStatusEvent != null)
            {
                this.DownloadStatusEvent(PercentComplete, BytesTransferred, TotalBytes);
            }
        }

        private HttpWebRequest GetWebRequest()
        {
            HttpWebRequest request2 = (HttpWebRequest) WebRequest.Create(this.m_strURL);
            if (StringType.StrCmp(this.m_strCookie, "", false) != 0)
            {
                request2.Headers.Add("Cookie", this.m_strCookie);
            }
            request2.Method = this.m_strMethod;
            if (StringType.StrCmp(this.m_strMethod, "POST", false) == 0)
            {
                request2.ContentType = "application/x-www-form-urlencoded";
                StreamWriter writer = new StreamWriter(request2.GetRequestStream());
                writer.Write(this.m_strPostData);
                writer.Close();
                writer = null;
            }
            return request2;
        }

        private void GotData(IAsyncResult ar)
        {
            HTTPDownloadState asyncState = (HTTPDownloadState) ar.AsyncState;
            if (this._cancel)
            {
                this.Close(asyncState);
            }
            else
            {
                int count = asyncState.Inputstream.EndRead(ar);
                if (count > 0)
                {
                    asyncState.Outputstream.Write(asyncState.Data, 0, count);
                    HTTPDownloadState state2 = asyncState;
                    state2.BytesRead += count;
                }
                int percentComplete = asyncState.PercentComplete();
                int bytesRead = asyncState.BytesRead;
                int totalBytes = asyncState.TotalBytes;
                if (count > 0)
                {
                    asyncState.Inputstream.BeginRead(asyncState.Data, 0, asyncState.Data.Length, new AsyncCallback(this.GotData), asyncState);
                }
                else
                {
                    this.Close(asyncState);
                    totalBytes = bytesRead;
                }
                try
                {
                    this._SynchronisingObject.Invoke(new DownloadStatusDelegate(this.DownloadStatusSub), new object[] { percentComplete, bytesRead, totalBytes });
                }
                catch (Exception exception1)
                {
                    ProjectData.SetProjectError(exception1);
                    if (this.DownloadStatusEvent != null)
                    {
                        this.DownloadStatusEvent(percentComplete, bytesRead, totalBytes);
                    }
                    ProjectData.ClearProjectError();
                }
            }
        }

        protected virtual void GotFilenameSub(string strFilename)
        {
            if (this.GotFilenameEvent != null)
            {
                this.GotFilenameEvent(strFilename);
            }
        }

        private string ResponseFilename(HttpWebResponse res)
        {
            string responseHeader = res.GetResponseHeader("Content-Disposition");
            int index = responseHeader.IndexOf("filename=");
            if (index != -1)
            {
                responseHeader = responseHeader.Substring(index + 9);
            }
            return responseHeader;
        }        
    }
}

