﻿namespace TorrentHarvester
{
    using Microsoft.VisualBasic;
    using Microsoft.VisualBasic.CompilerServices;
    using System;
    using System.Data;
    using System.Runtime.InteropServices;

    public class CField
    {
        private string _strRow;
        public int Column;
        private string Field_Type_Date;
        private string Field_Type_Filesize;
        private string Field_Type_Number;
        private string Field_Type_Text;
        public bool Hidden;
        public bool InheritIfBlank;
        private const string Key_Field = "field";
        private const string Key_Field_Column = "column";
        private const string Key_Field_Hidden = "hidden";
        private const string Key_Field_Inherit = "inherit";
        private const string Key_Field_Name = "name";
        private const string Key_Field_Row = "row";
        private const string Key_Field_Type = "type";
        public string Name;
        public CRow Row;
        public CTextTransforms Transforms;
        public EFieldType Type;

        public CField()
        {
            this.Field_Type_Text = "text";
            this.Field_Type_Number = "number";
            this.Field_Type_Filesize = "filesize";
            this.Field_Type_Date = "date";
            this.Transforms = new CTextTransforms();
            this.Type = EFieldType.ftText;
        }

        public CField(CXMLNode oXMLNode, CRow oRow)
        {
            this.Field_Type_Text = "text";
            this.Field_Type_Number = "number";
            this.Field_Type_Filesize = "filesize";
            this.Field_Type_Date = "date";
            this.Transforms = new CTextTransforms();
            this.Load(oXMLNode);
            this.Row = oRow;
        }

        public void Load(CXMLNode oXMLNode)
        {
            this.Name = oXMLNode.get_StringAttribute("name", "");
            string sLeft = oXMLNode.get_StringAttribute("type", "");
            if (StringType.StrCmp(sLeft, this.Field_Type_Text, false) == 0)
            {
                this.Type = EFieldType.ftText;
            }
            else if (StringType.StrCmp(sLeft, this.Field_Type_Number, false) == 0)
            {
                this.Type = EFieldType.ftNumber;
            }
            else if (StringType.StrCmp(sLeft, this.Field_Type_Filesize, false) == 0)
            {
                this.Type = EFieldType.ftFilesize;
            }
            else if (StringType.StrCmp(sLeft, this.Field_Type_Date, false) == 0)
            {
                this.Type = EFieldType.ftDate;
            }
            else
            {
                this.Type = EFieldType.ftText;
            }
            this.Column = oXMLNode.get_IntegerAttribute("column", 0);
            this.InheritIfBlank = oXMLNode.get_BooleanAttribute("inherit", false);
            this.Hidden = oXMLNode.get_BooleanAttribute("hidden", false);
            this.Transforms = new CTextTransforms(oXMLNode);
        }

        public void Save(CXMLNode oXMLNode)
        {
            oXMLNode = oXMLNode.Create("field");
            CXMLNode node = oXMLNode;
            node.AddAttribute("name", this.Name, false);
            switch (this.Type)
            {
                case EFieldType.ftText:
                    node.AddAttribute("type", this.Field_Type_Text, false);
                    break;

                case EFieldType.ftNumber:
                    node.AddAttribute("type", this.Field_Type_Number, false);
                    break;

                case EFieldType.ftFilesize:
                    node.AddAttribute("type", this.Field_Type_Filesize, false);
                    break;

                case EFieldType.ftDate:
                    node.AddAttribute("type", this.Field_Type_Date, false);
                    break;
            }
            node.AddAttribute("column", this.Column.ToString(), false);
            node.AddAttribute("inherit", Interaction.IIf(this.InheritIfBlank, "Y", "").ToString(), false);
            node.AddAttribute("hidden", Interaction.IIf(this.Hidden, "Y", "").ToString(), false);
            node = null;
            this.Transforms.Save(oXMLNode);
        }

        public override string ToString()
        {
            return this.Name;
        }

        public string Transform(DataRow row, [Optional, DefaultParameterValue(1)] int intExtraColumns)
        {
            string str;
            if (this.Column == 0)
            {
                str = "";
            }
            else
            {
                str = this.Transforms.Transform(row[(this.Column + intExtraColumns) - 1].ToString());
            }
            return str.Trim();
        }

        public string ColumnKey
        {
            get
            {
                return (this.Name + ((int) this.Type).ToString());
            }
        }

        public string GlobalKey
        {
            get
            {
                return (this.Name + "~" + this.Row.Name);
            }
        }

        public string RowKey
        {
            get
            {
                return this.Name;
            }
        }

        public enum EFieldType
        {
            ftDate = 3,
            ftFilesize = 2,
            ftNumber = 1,
            ftText = 0,
            ftUnknown = -1
        }
    }
}

