﻿namespace TorrentHarvester
{
    using Microsoft.VisualBasic;
    using Microsoft.VisualBasic.CompilerServices;
    using System;
    using System.Collections;
    using System.ComponentModel;
    using System.Runtime.CompilerServices;
    using System.Threading;

    public class CMultiSearch
    {
        private Collection _oRunningThreads;
        private Collection _oWaitingEngines;
        private ISynchronizeInvoke _SynchronisingObject;

        public event EngineStatusChangedEventHandler EngineStatusChanged;

        public event FileStatusChangedEventHandler FileStatusChanged;

        public event FinishedEventHandler Finished;

        public event PageStatusChangedEventHandler PageStatusChanged;

        public CMultiSearch(ISynchronizeInvoke objSynchronisingObject)
        {
            this._SynchronisingObject = objSynchronisingObject;
        }

        private void CreateEngineStatusChangedEventHandler(CEngine oEngine)
        {
            oEngine.EngineStatusChanged += new CEngine.EngineStatusChangedEventHandler(this.Engine_EngineStatusChanged);
        }

        private void CreateFileStatusChangedEventHandler(CEngine oEngine)
        {
            oEngine.FileStatusChanged += new CEngine.FileStatusChangedEventHandler(this.Engine_FileStatusChanged);
        }

        private void CreatePageStatusChangedEventHandler(CEngine oEngine)
        {
            oEngine.PageStatusChanged += new CEngine.PageStatusChangedEventHandler(this.Engine_PageStatusChanged);
        }

        private void Engine_EngineStatusChanged(CEngine oEngine)
        {
            if (((oEngine.Status == CEngine.EEngineStatus.esError) | (oEngine.Status == CEngine.EEngineStatus.esOK)) | (oEngine.Status == CEngine.EEngineStatus.esStopped))
            {
                try
                {
                    Collection collection = this._oRunningThreads;
                    lock (collection)
                    {
                        this._oRunningThreads.Remove(oEngine.Name);
                    }
                    this.StartWaitingThreads();
                }
                catch (Exception exception1)
                {
                    ProjectData.SetProjectError(exception1);
                    ProjectData.ClearProjectError();
                }
            }
            this._SynchronisingObject.Invoke(new EngineStatusChangedDelegate(this.EngineStatusChangedSub), new object[] { oEngine });
        }

        private void Engine_FileStatusChanged(CFile oFile)
        {
            if (StringType.StrCmp(oFile.get_FieldText("Title"), "", false) != 0)
            {
                this._SynchronisingObject.Invoke(new FileStatusChangedDelegate(this.FileStatusChangedSub), new object[] { oFile });
            }
        }

        private void Engine_PageStatusChanged(CPage oPage)
        {
            this._SynchronisingObject.Invoke(new PageStatusChangedDelegate(this.PageStatusChangedSub), new object[] { oPage });
        }

        private void EngineStatusChangedSub(CEngine oEngine)
        {
            if (this.EngineStatusChangedEvent != null)
            {
                this.EngineStatusChangedEvent(oEngine);
            }
        }

        private void FileStatusChangedSub(CFile oFile)
        {
            if (this.FileStatusChangedEvent != null)
            {
                this.FileStatusChangedEvent(oFile);
            }
        }

        public void Halt()
        {
            Collection collection = this._oRunningThreads;
            lock (collection)
            {
                IEnumerator enumerator;
                while (this._oWaitingEngines.Count != 0)
                {
                    CEngine oEngine = (CEngine) this._oWaitingEngines[this._oWaitingEngines.Count];
                    this._oWaitingEngines.Remove(this._oWaitingEngines.Count);
                    oEngine.Status = CEngine.EEngineStatus.esStopped;
                    if (this.EngineStatusChangedEvent != null)
                    {
                        this.EngineStatusChangedEvent(oEngine);
                    }
                }
                try
                {
                    enumerator = this._oRunningThreads.GetEnumerator();
                    while (enumerator.MoveNext())
                    {
                        ((Thread) enumerator.Current).Abort();
                    }
                }
                finally
                {
                    if (enumerator is IDisposable)
                    {
                        ((IDisposable) enumerator).Dispose();
                    }
                }
            }
        }

        private void PageStatusChangedSub(CPage oPage)
        {
            if (this.PageStatusChangedEvent != null)
            {
                this.PageStatusChangedEvent(oPage);
            }
        }

        public void Start(Collection oEngines, string strSearchText)
        {
            this._oRunningThreads = new Collection();
            this._oWaitingEngines = oEngines;
            foreach (CEngine engine in this._oWaitingEngines)
            {
                engine.CurrentSearchText = strSearchText;
                engine.Status = CEngine.EEngineStatus.esWaiting;
                if (this.EngineStatusChangedEvent != null)
                {
                    this.EngineStatusChangedEvent(engine);
                }
            }
            this.StartWaitingThreads();
        }

        public void StartWaitingThreads()
        {
            Collection collection = this._oRunningThreads;
            lock (collection)
            {
                while ((this._oWaitingEngines.Count != 0) & (this._oRunningThreads.Count < modGlobal.g_oSettings.MaxConcurrentSearches))
                {
                    CEngine oEngine = (CEngine) this._oWaitingEngines[1];
                    this._oWaitingEngines.Remove(oEngine.Name);
                    this.CreateFileStatusChangedEventHandler(oEngine);
                    this.CreatePageStatusChangedEventHandler(oEngine);
                    this.CreateEngineStatusChangedEventHandler(oEngine);
                    Thread item = new Thread(new ThreadStart(oEngine.Search));
                    item.Name = oEngine.Name;
                    this._oRunningThreads.Add(item, item.Name, null, null);
                    item.Start();
                }
                if (((this._oWaitingEngines.Count + this._oRunningThreads.Count) == 0) && (this.FinishedEvent != null))
                {
                    this.FinishedEvent();
                }
            }
        }

        private delegate void EngineStatusChangedDelegate(CEngine oEngine);

        public delegate void EngineStatusChangedEventHandler(CEngine oEngine);

        private delegate void FileStatusChangedDelegate(CFile oFile);

        public delegate void FileStatusChangedEventHandler(CFile oFile);

        public delegate void FinishedEventHandler();

        public delegate void PageStatusChangedDelegate(CPage oPage);

        public delegate void PageStatusChangedEventHandler(CPage oPage);
    }
}

